@@ -18,7 +18,7 @@
  */
 
 /***************************************************************************
- * Copyright (C) 2017-2019 ZmartZone IAM
+ * Copyright (C) 2017-2020 ZmartZone IAM
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -422,10 +422,11 @@ int oidc_authz_worker22(request_rec *r, const json_t * const claims,
  * Apache >=2.4 authorization routine: match the claims from the authenticated user against the Require primitive
  */
 authz_status oidc_authz_worker24(request_rec *r, const json_t * const claims,
-		const char *require_args, oidc_authz_match_claim_fn_type match_claim_fn) {
+		const char *require_args, const void *parsed_require_args, oidc_authz_match_claim_fn_type match_claim_fn) {
 
 	int count_oauth_claims = 0;
-	const char *t, *w;
+	const char *t, *w, *err = NULL;
+	const ap_expr_info_t *expr = parsed_require_args;
 
 	/* needed for anonymous authentication */
 	if (r->user == NULL)
@@ -435,8 +436,17 @@ authz_status oidc_authz_worker24(request_rec *r, const json_t * const claims,
 	if (!claims)
 		return AUTHZ_DENIED;
 
+	if (expr) {
+		t = ap_expr_str_exec(r, expr, &err);
+		if (err) {
+			oidc_error(r, "could not evaluate expression '%s': %s", require_args, err);
+			return AUTHZ_DENIED;
+		}
+	} else {
+		t = require_args;
+	}
+
 	/* loop over the Required specifications */
-	t = require_args;
 	while ((w = ap_getword_conf(r->pool, &t)) && w[0]) {
 
 		count_oauth_claims++;
