@@ -18,7 +18,7 @@
  */
 
 /***************************************************************************
- * Copyright (C) 2017-2019 ZmartZone IAM
+ * Copyright (C) 2017-2020 ZmartZone IAM
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -275,7 +275,7 @@ static apr_byte_t oidc_session_save_cookie(request_rec *r, oidc_session_t *z,
 							(first_time ?
 									OIDC_COOKIE_EXT_SAME_SITE_LAX :
 									OIDC_COOKIE_EXT_SAME_SITE_STRICT) :
-									NULL);
+									OIDC_COOKIE_EXT_SAME_SITE_NONE);
 
 	return TRUE;
 }
